@@ -39,7 +39,6 @@
 
 #include "i18n.h"
 #include "util.h"
-#include "exechelp.h"
 #include "sysutils.h"
 #include "status.h"
 #include "membuf.h"
@@ -54,9 +53,9 @@
 
 /* The time we wait until the agent or the dirmngr are ready for
    operation after we started them before giving up.  */
-#define SECS_TO_WAIT_FOR_AGENT 5
-#define SECS_TO_WAIT_FOR_KEYBOXD 5
-#define SECS_TO_WAIT_FOR_DIRMNGR 5
+#define SECS_TO_WAIT_FOR_AGENT 8
+#define SECS_TO_WAIT_FOR_KEYBOXD 8
+#define SECS_TO_WAIT_FOR_DIRMNGR 8
 
 /* A bitfield that specifies the assuan categories to log.  This is
    identical to the default log handler of libassuan.  We need to do
@@ -129,6 +128,21 @@ set_libassuan_log_cats (unsigned int newcats)
 }
 
 
+/* Get the last Windows error from an Assuan socket function and print
+ * the raw error code using log_info.  */
+void
+log_libassuan_system_error (assuan_fd_t fd)
+{
+  int w32err = 0;
+
+  if (assuan_sock_get_flag (fd, "w32_error", &w32err))
+    w32err = -1;  /* Old Libassuan or not Windows.  */
+
+  if (w32err != -1)
+    log_info ("system error code: %d (0x%x)\n", w32err, w32err);
+}
+
+
 
 static gpg_error_t
 send_one_option (assuan_context_t ctx, gpg_err_source_t errsource,
@@ -513,8 +527,6 @@ start_new_service (assuan_context_t *r_ctx,
       i = 0;
       argv[i++] = "--homedir";
       argv[i++] = abs_homedir;
-      if (module_name_id == GNUPG_MODULE_NAME_AGENT)
-        argv[i++] = "--use-standard-socket";
       if (program_arg)
         argv[i++] = program_arg;
       argv[i++] = "--daemon";
@@ -524,10 +536,10 @@ start_new_service (assuan_context_t *r_ctx,
           && assuan_socket_connect (ctx, sockname, 0, connect_flags))
         {
 #ifdef HAVE_W32_SYSTEM
-          err = gnupg_process_spawn (program? program : program_name, argv,
-                                     GNUPG_PROCESS_DETACHED, NULL, NULL);
+          err = gpgrt_process_spawn (program? program : program_name, argv,
+                                     GPGRT_PROCESS_DETACHED, NULL, NULL);
 #else /*!W32*/
-          err = gnupg_process_spawn (program? program : program_name, argv,
+          err = gpgrt_process_spawn (program? program : program_name, argv,
                                      0, NULL, NULL);
 #endif /*!W32*/
           if (err)
